/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#include <gl/glee.h>
#include <gl/glu.h>

#include "../../_common/Shader.h"
#include "../../_common/DisplacementMap.h"
#include "../../_common/PrimitiveFactory.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 5 - Displacement Mapping";

const int MGL_WINDOW_WIDTH = 1024;
const int MGL_WINDOW_HEIGHT = 768;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

	// Called by message handler
	// x and y are absolute client coordinates
	void MouseDownPosition( int x, int y );

private:

	double m_dElapTime;						// accumulated time
	int m_nWindowWidth, m_nWindowHeight;	// window size

	Primitive *m_pTerrain;					// the patch to displace
	Primitive *m_pPaintSurface;				// quad to display the highmap

	CDisplacementMap m_HeightMap;			// our displacement map

	GLuint m_iVertShad;						// displacement vertex shader handle

};

#endif // MGLAPP_H_INCLUDED