/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#include <gl/glee.h>
#include <gl/glu.h>
#include "../../_common/PrimitiveFactory.h"
#include "../../_common/Shader.h"
#include "../../_common/MGLTargaImage.h"
#include "../../_common/MGLVector.h"

using namespace MGL;


// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 5 - Parallax Mapping";

const int MGL_WINDOW_WIDTH = 1024;
const int MGL_WINDOW_HEIGHT = 768;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

	// Called by message handler when left mouse button is down
	// x and y are relative client coordinates
	void LeftMouseMove( int dx, int dy );

	// Called by message handler when right mouse button is down
	// x and y are relative client coordinates
	void RightMouseMove( int dx, int dy );

private:

	double m_dElapTime;						// accumulated time
	int m_nWindowWidth, m_nWindowHeight;	// window size

	float m_fRotTheta, m_fRotPhi;				// accumulated movements of mouse
	float m_fLightX, m_fLightZ;				

	Primitive *m_pQuad;
	Vector3 *m_pTangentData;

	GLhandleARB m_hVertShad;
	GLhandleARB m_hFragShad;
	GLhandleARB m_hShadProg;

	GLuint m_hColorTex;
	GLuint m_hNormBumpTex;

};

#endif // MGLAPP_H_INCLUDED