#include "CProjector.h"

namespace MGL{

CProjector :: CProjector()
{
	float identity[16] = {	1, 0, 0, 0, 
							0, 1, 0, 0, 
							0, 0, 1, 0, 
							0, 0, 0, 1 };
	mat4_view = mat4(identity);
	mat4_projection = mat4(identity);
}

void CProjector :: LookAt(Vector3 pos, Vector3 at, Vector3 v_up)
{
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	
	gluLookAt(pos.x,  pos.y,  pos.z,
			  at.x,   at.y,   at.z,
			  v_up.x, v_up.y, v_up.z);
	
	glGetFloatv(GL_MODELVIEW_MATRIX, (GLfloat*)&mat4_view);

	glPopMatrix();
}

void CProjector :: SetProjection(float aspect_ratio_xy, float fovy)
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	
	gluPerspective(fovy, aspect_ratio_xy, 1, 100);
	glGetFloatv(GL_PROJECTION_MATRIX, (GLfloat*)&mat4_projection);

	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
}

void CProjector :: SetupTexGen()
{
	float offset[16] = { 0.5f,	0,	0,	0,
						 0,	0.5f,	0,	0, 
						 0,		0,	0,	0,
						 0.5f,0.5f, 0.0f, 1 };
	
	mat4 m_offset = mat4(offset);

	mat4 m_result = (m_offset * mat4_projection * mat4_view).transpose(); 
	
	glTexGenfv(GL_S, GL_EYE_PLANE, (GLfloat*)&m_result.mat[0]);
	glTexGenfv(GL_T, GL_EYE_PLANE, (GLfloat*)&m_result.mat[4]);
	glTexGenfv(GL_R, GL_EYE_PLANE, (GLfloat*)&m_result.mat[8]);
	glTexGenfv(GL_Q, GL_EYE_PLANE, (GLfloat*)&m_result.mat[12]);
	
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
	
	glEnable(GL_TEXTURE_GEN_S);	
	glEnable(GL_TEXTURE_GEN_T);	
	glEnable(GL_TEXTURE_GEN_R);
	glEnable(GL_TEXTURE_GEN_Q);
}


void CProjector :: SetupTexMatrix()
{
	float offset[16] = { 0.5f,	0,	0,	0,
				 0,	0.5f,	0,	0, 
				 0,		0,	0,	0,
				 0.5f,0.5f, 0.0f, 1 };
	
	mat4 m_offset = mat4(offset);
	mat4 m_result = m_offset * mat4_projection * mat4_view; 
	
	glMatrixMode(GL_TEXTURE);
	glLoadMatrixf((GLfloat*) &m_result);
	glMatrixMode(GL_MODELVIEW);
}

} // namespace MGL

