/*************************************************************************
MGLApp.h

Demo for "Projective Textures" section

Author   :   Andrei Gradinari
Based on Dave Astle's application template

Written for More OpenGL Game Programming
August, 2005
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#ifdef WIN32
  #define WIN32_LEAN_AND_MEAN
  #define WIN32_EXTRA_LEAN
  #include <windows.h>
#endif

#include <gl/glee.h>
#include <gl/glu.h>

#include "../../_common/MGLTargaImage.h"
#include "../../_common/MGLTimer.h"
#include "../../_common/MGLTypes.h"
#include "../../_common/MGLVector.h"
#include "../../_common/MGLUtil.h"
#include "../../_common/prim.h"

#include "CProjector.h"

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter Advanced Texture Mapping - Projective Textures Demo";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;
const bool MGL_FORCE_FULLSCREEN = false;
extern bool g_bUseShaders;

using namespace MGL;

class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

private:
	
	void SetupLighting();
	void RenderGeometry(bool b_first_pass = true);
	void LookAt(float, float, float, float, float, float, float, float, float);
	void UpdateShaderUniforms();
	 
	static GLuint LoadTexture(char* fname, GLenum wrapst = GL_REPEAT, bool mipmap = true);
	static GLhandleARB LoadShader(const char* s_vert, const char* s_frag);
	bool LoadData(); // load models, shaders and textures
		
	double m_dElapTime;
	int m_nWindowWidth, m_nWindowHeight;
	
	Vector3 lamp_pos_initial[3]; // initial light pos
	Vector3 lamp_pos[3]; // initial light pos
	float lamp_alpha, lamp_beta; // alpha and beta angles
	
	CProjector m_Projector;

	mat4 mat4_view, mat4_view_inv; 
	// View matrix and inverse stored in row major order. Updated each time LookAt func called
	
	GLhandleARB m_iProjShader;
	
	GLuint m_iFloorTexture, m_iProjTexture;
	Primitive *m_pMesh;
	Primitive *m_pLampMesh[2];
};

#endif // MGLAPP_H_INCLUDED