/*************************************************************************
MGLApp.h

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifndef MGLAPP_H_INCLUDED
#define MGLAPP_H_INCLUDED

#include <gl/glee.h>
#include <gl/glu.h>
#include <math.h>
#include <stdio.h>

// values used to set up the window and rendering context
const char MGL_APP_NAME[]	= "MOGLGP - Chapter 5 - Render To Texture 2";

const int MGL_WINDOW_WIDTH = 800;
const int MGL_WINDOW_HEIGHT = 600;

const int MGL_COLOR_BITS = 32;
const int MGL_DEPTH_BITS = 32;
const int MGL_STENCIL_BITS = 0;

const bool MGL_FORCE_FULLSCREEN = false;


class MGLApp
{
public:
	MGLApp();
	virtual ~MGLApp();

	bool Init();
	bool Shutdown();
	void SetupProjection( int width, int height );
	void Update( double dt );
	void Render();

	// Draws an dodecahedron
	void DrawDodecahedron();

	// Draws a teapot
	void DrawTeapot();

	// Called by message handler when left mouse button is down
	// x and y are relative client coordinates
	void LeftMouseMove( int dx, int dy );

	// Called by message handler when right mouse button is down
	// x and y are relative client coordinates
	void RightMouseMove( int dx, int dy );

	// Keeps track of framerate and draws it to the screen
	void DrawFPS();

private:

	double m_dElapTime;						// accumulated time
	int m_nWindowWidth, m_nWindowHeight;	// window size

	float m_fLeftX, m_fLeftY;				// accumulated movements of mouse
	float m_fRightX, m_fRightY;				

	int m_nFrames;							// framerate info
	double m_dFPSTime, m_dFPS;

};

#endif // MGLAPP_H_INCLUDED