#define WIN32_LEAN_AND_MEAN
#define WIN32_EXTRA_LEAN
#include <windows.h>
#include "timer.h"


//////////////////////////////////////////////////////////////////////////
// CTimer class implementation
//////////////////////////////////////////////////////////////////////////
void CTimer::Start()
{
	static __int64 time;

	if (!stopped)
		return;

	if (reset)
	{
		QueryPerformanceCounter((LARGE_INTEGER*)&startTime);
		savedTime = lastTime = stopTime = startTime;
		reset = false;
	}else
	{
		QueryPerformanceCounter((LARGE_INTEGER*)&time);

		lastTime += (time - stopTime);
		startTime += (time - stopTime);
	}

	stopped = false;
}

void CTimer::Stop()
{
	if (stopped)
		return;

	stopped = true;
	QueryPerformanceCounter((LARGE_INTEGER*)&stopTime);
}

void CTimer::Reset()
{
	reset = stopped = true;
	stopTime = lastTime = savedTime = startTime = 0;
}

double CTimer::GetTime() const
{
	static __int64 endTime;

	if (reset)		return 0.0;
	if (stopped)	return (double)(stopTime-startTime)*rate_inv;

	QueryPerformanceCounter((LARGE_INTEGER*)&endTime);


	return (double)(endTime-startTime)*rate_inv;
}

double CTimer::GetElapsedTime()
{
	static __int64 temp;

	temp = lastTime;

	if (stopped)
	{
		double result = (double)(stopTime-lastTime)*rate_inv;
		lastTime = stopTime;
		return result;
	}

	QueryPerformanceCounter((LARGE_INTEGER*)&lastTime);

	return (double)(lastTime - temp)*rate_inv;
}

CTimer::CTimer()
{
	__int64 rate;

	initialized = false;

	if(!QueryPerformanceFrequency((LARGE_INTEGER*)&rate))
		return;
	
	if(!rate)
		return;
	
	rate_inv=1.0/(double)rate;

	initialized = true;
	Reset();
}

CTimer::~CTimer()
{
}
