#include "shader.h"
#include <stdio.h>

static long GetFileLength( FILE *pFile )
{
	long nSaved = ftell( pFile ), nSize;
	fseek( pFile, 0, SEEK_END );
	nSize = ftell( pFile );
	fseek( pFile, nSaved, SEEK_SET );
	return nSize;
}

GLhandleARB LoadGLSLShader( GLenum iShadType, const char *szSourceFile, char *szErrLog, int *piErrLogLen )
{
	GLhandleARB iHandle = 0;

	FILE *pInFile = fopen( szSourceFile, "rt" );
	if( pInFile == NULL )
		return false;

	// read file into string
	GLint iLen = (GLint)GetFileLength( pInFile );
	GLcharARB *szShader = new char[ iLen + 1 ];

	if( iLen == 0 || szShader == NULL )
		return NULL;

	iLen = (GLint)fread( szShader, sizeof(GLcharARB), iLen, pInFile );
	szShader[iLen] = '\0';
	fclose( pInFile );

	
	// load and compile
	iHandle = glCreateShaderObjectARB( iShadType );
	glShaderSourceARB( iHandle, 1, (const GLcharARB**)&szShader, &iLen );
	glCompileShaderARB( iHandle );
	delete[] szShader;

	// return info log
	if( szErrLog && piErrLogLen )
	{
		int iTotalLen = 0, nCharsWritten = 0;

		glGetObjectParameterivARB( iHandle, GL_OBJECT_INFO_LOG_LENGTH_ARB,
			&iTotalLen );

		if( iTotalLen > 1 )
			glGetInfoLogARB( iHandle, *piErrLogLen, &nCharsWritten, szErrLog );

		*piErrLogLen = iTotalLen - 1;
	}

	return iHandle;
}

GLuint LoadASMShader( GLenum iShadType, const char *szSourceFile, char **pszErrLog, int *piErrLine )
{
	GLuint iHandle = 0;

	FILE *pInFile = fopen( szSourceFile, "rt" );
	if( pInFile == NULL )
		return 0;

	// read file into string
	GLint iLen = (GLint)GetFileLength( pInFile );
	GLcharARB *szShader = new char[ iLen + 1 ];

	if( iLen == 0 || szShader == NULL )
		return 0;

	iLen = (GLint)fread( szShader, sizeof(GLcharARB), iLen, pInFile );
	szShader[iLen] = '\0';
	fclose( pInFile );

	// load and compile
	glGenProgramsARB( 1, &iHandle );
	glBindProgramARB( iShadType, iHandle );
	glProgramStringARB( iShadType, GL_PROGRAM_FORMAT_ASCII_ARB, (GLsizei)iLen, (GLubyte*)szShader );
	delete[] szShader;

	if( piErrLine )
		glGetIntegerv( GL_PROGRAM_ERROR_POSITION_ARB, piErrLine );

	if( pszErrLog )
		*pszErrLog = (char*)glGetString( GL_PROGRAM_ERROR_STRING_ARB );

	return iHandle;
}
