/*************************************************************************
MGLApp.cpp

Main application class.

Author   :   Dave Astle

Written for More OpenGL Game Programming
*************************************************************************/

#ifdef WIN32
	#define WIN32_LEAN_AND_MEAN
	#define WIN32_EXTRA_LEAN
	#include <windows.h>
#endif

#include "MGLApp.h"
#include "..//..//_common//MGLTargaImage.h"
#include <cstdio>

MGLApp::MGLApp()
{
}

MGLApp::~MGLApp()
{
}

bool MGLApp::Init()
{	
	if (!GLeeInit())
		return false;

  glGenTextures(2, m_textures);
  MGL::TargaImage image;

  image.Load("uncompressed.tga");

  glBindTexture(GL_TEXTURE_2D, m_textures[0]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());

  GLint width, height, size;
  GLenum compressedFormat;
  GLubyte *pData = NULL;

  glBindTexture(GL_TEXTURE_2D, m_textures[1]);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

  // Try to load the compressed image. If it doesn't exist, have the OpenGL driver compress it
  // and save the results for future use.
  if (LoadCompressedImage(width, height, compressedFormat, size, pData))
  {
    glCompressedTexImage2D(GL_TEXTURE_2D, 0, compressedFormat, width, height, 0, size, pData);
    delete[] pData;
    pData = NULL;
  }
  else
  {
    glHint(GL_TEXTURE_COMPRESSION_HINT, GL_NICEST);
    glTexImage2D(GL_TEXTURE_2D, 0, GL_COMPRESSED_RGB, image.GetWidth(), image.GetHeight(), 0, image.GetFormat(), image.GetType(), image.GetImage());

    GLint wasCompressed;
    glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_COMPRESSED, &wasCompressed);
    if (wasCompressed)
    {
      GLint compressedSize, format;
      glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_COMPRESSED_IMAGE_SIZE, &compressedSize);
      glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_INTERNAL_FORMAT, &format);
      GLubyte *pCompressedData = new GLubyte[compressedSize];
      glGetCompressedTexImage(GL_TEXTURE_2D, 0, pCompressedData);

      SaveCompressedImage(image.GetWidth(), image.GetHeight(), format, compressedSize, pCompressedData);

      delete[] pCompressedData;
    }
  }

  image.Release();

  glEnable(GL_TEXTURE_2D);

	return true;
}

bool MGLApp::Shutdown()
{
	return true;
}

void MGLApp::SetupProjection(int width, int height)
{
	// avoid div by zero
	if( height == 0 )
		height = 1;					

	// set up GL viewport
	glViewport( 0, 0, width, height );

	// set up a perspective projection
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective( 45.0f, (GLfloat)width/(GLfloat)height, 1.0f, 1000.0f );
}

void MGLApp::Update( double dt )
{
}

void MGLApp::Render()
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

	// set up the camera
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

  // draw the normal image
  glBindTexture(GL_TEXTURE_2D, m_textures[0]);
  glBegin(GL_QUADS);
    glTexCoord2i(0, 0); glVertex3f(-2.1f, -1.0f, -4.0f);
    glTexCoord2i(0, 1); glVertex3f(-2.1f, 1.0f, -4.0f);
    glTexCoord2i(1, 1); glVertex3f(-0.1f, 1.0f, -4.0f);
    glTexCoord2i(1, 0); glVertex3f(-0.1f, -1.0f, -4.0f);
  glEnd();

  // draw the compressed image
  glBindTexture(GL_TEXTURE_2D, m_textures[0]);
  glBegin(GL_QUADS);
    glTexCoord2i(0, 0); glVertex3f(0.1f, -1.0f, -4.0f);
    glTexCoord2i(0, 1); glVertex3f(0.1f, 1.0f, -4.0f);
    glTexCoord2i(1, 1); glVertex3f(2.1f, 1.0f, -4.0f);
    glTexCoord2i(1, 0); glVertex3f(2.1f, -1.0f, -4.0f);
  glEnd();
}

/* The image file format will be as follows:
    Bytes   Data
    ----------------
    0-3     width
    4-7     height
    8-11    format
    12-15   size
    16+     compressed image data
*/


GLboolean MGLApp::LoadCompressedImage(GLint &width, GLint &height, GLenum &compressedFormat, GLint &size, GLubyte *&pData)
{
  FILE *pFile = fopen("compressed.mgl", "rb");
  if (!pFile)
    return GL_FALSE;
  GLuint info[4];

  fread(info, 4, 4, pFile);
  width = info[0];
  height = info[1];
  compressedFormat = info[2];
  size = info[3];

  pData = new GLubyte[size];
  fread(pData, size, 1, pFile);
  fclose(pFile);
  return GL_TRUE;
}


void MGLApp::SaveCompressedImage(GLint width, GLint height, GLenum compressedFormat, GLint size, GLubyte *pData)
{
  FILE *pFile = fopen("compressed.mgl", "wb");
  if (!pFile)
    return;

  GLuint info[4];

  info[0] = width;
  info[1] = height;
  info[2] = compressedFormat;
  info[3] = size;

  fwrite(info, 4, 4, pFile);
  fwrite(pData, size, 1, pFile);
  fclose(pFile);
}